/*--------------------------------------------------------------*
 * ASA2HTML REXX     A REXX filter to convert a listing file    *
 *     (with ASA control chars) to plain text, with special     *
 *     handling for HTML conversion:                            *
 *     + lines with ___ -> <i> ... </i>                         *
 *     <,%,> in formulas -> {,~,}                               *
 *                                                              *
 * Usage:                                                       *
 *      ASA2HTML                                                *
 *                                                              *
 *--------------------------------------------------------------*
 *  Author:  Michael Friendly       <FRIENDLY@VM1.YorkU.CA>     *
 * Revised:  17 Aug 1995 23:30:03                               *
 * Version:  1.2                                                *
 *   Remove initial blank lines                                 *
 *   Translate <, > inside formulas to {,}                      *
 *--------------------------------------------------------------*/
 
signal on error
lines=0
hexin = 'BFAFFA'x      ||,     /* underline, bullet, vert bar */
        'ACBCABBB'x    ||,     /* box corners                 */
        'CCCBEBEC'x
hexout= '-*|' || '++++++++'
 
/* 'readto record'   */             /* remove first 1 record */
formula = 0
code=0
do forever
  'readto record'
  parse var record 1 cc 2 rest
  rest=translate(rest,hexout,hexin)
 
  /* translate <, %, > inside formulas */
/*if index(rest,'<!-- eDF')>0 then formula=0
  if formula & substr(rest,1,1)^='<' ,
     then rest=translate(rest,'{ }','<%>')
  if index(rest,'<!-- DF')>0 then formula=1
*/
  if index(rest,'</code>')>0 then code=0
  if code then do
     lt = index(rest,'<')
     if lt>0 & substr(rest,lt+1,1) ^= ' ' then do
        rest = left(rest,lt-1)||'&lt;'||substr(rest,lt+1)
        end
     end
  if index(rest,'<code>' )>0 then code=1
 
  if cc ^='+' then do         /* handle 'italics' */
     signal off error         /* don't crap out on eof */
     'peekto next'
     signal on error
     if rc=0 then do
     parse var next 1 cn 2 overlay
     if cn='+' & index(overlay,'_')>0 then do
        save = rest
        rest = ''
        do while(overlay ^= ' ')
           l1 = index(overlay,'_')
           if l1=0 then leave
           l2 = verify(overlay,'_',,l1+1)
           if l2=0 then l2=length(overlay)
           rest = rest||substr(save,1,l1-1)||'<i>'
           rest = rest||substr(save,l1,l2-l1+1)||'</i>'
           save = substr(overlay,l2+1)
           overlay = substr(overlay,l2+1)
           end
        rest = rest save
     end
     end
  end
 
  select
     when cc=' '  then do
        'output' rest
        lines = lines+1
        end
     when cc='+'  then iterate         /* ignore this record */
     when cc='0'  then do
        'output  '
        'output' rest
        lines = lines+2
        end
     when cc='-'  then do
     /* 'output  '  */
        'output  '
        'output' rest
        lines = lines+2
        end
 
     when cc='1'  then do             /* swallow page header */
        discard = 0
        do until index(record,'BFBF'x)>0 | discard>3
           'readto record'
           if index(record, '<', 2) > 0 then 'output' substr(record,2)
           discard = discard+1
           end
        end
     otherwise 'output' rest
     end
  end
 
Error:
   exit RC*(RC^=12)
